<?php
require_once ("album.php");
require_once ("artist.php");
class Database extends PDO {
	public function __construct() {
		parent::__construct ( "sqlite:" . __DIR__ . "/../music2.db" );
	}
	function getNumberOfAlbums() {
		$album_num = $this->query ( "SELECT count(*)  FROM album" );
		return $album_num->fetchColumn ();
	}
	/** 
	 * Functions used by the select page to sort full musci database 
	 */
	
	function getAlbumsByRank($num_returned = 25, $offset = 0) {
		return $this->getAlbumsByField ( "rank", $num_returned, $offset );
	}
	function getAlbumsByArtist($num_returned = 25, $offset = 0) {
		return $this->getAlbumsByField ( "artist_name", $num_returned, $offset );
	}
	function getAlbumsByYear($num_returned = 25, $offset = 0) {
		return $this->getAlbumsByField ( "year", $num_returned, $offset );
	}
	function getAlbumsByField($field, $num_returned = 25, $offset = 0) {
		$sql = "SELECT * FROM album NATURAL JOIN artist
	           ORDER BY $field ASC LIMIT $num_returned OFFSET $offset";
		$result = $this->query ( $sql );
		if ($result === FALSE) {
			// Only doing this for class. Would never do this in real life
			echo '<pre class="bg-danger">';
			print_r ( $this->errorInfo () );
			echo '</pre>';
			return array ();
		}
		$albums = array ();
		foreach ( $result as $row ) {
			$albums [] = Album::getAlbumFromRow ( $row );
		}
		return $albums;
	}
	/**
	 * Functions needed for the search example *
	 */
	function getNumberOfResults($query_term) {
		// This is what I usually do also referred to as escaping a string
		// However, doc page says we should be using the prepare function
		// instead
		$query_term = SQLite3::escapeString ( $query_term );
		$sql = "SELECT count(*) FROM album, artist WHERE album.artist_id = artist.artist_id
				AND (album.title LIKE '%$query_term%' 
					OR artist.artist_name LIKE '%$query_term%'
					OR album.year = '$query_term')";
		
		$result = $this->query ( $sql );
		return $result->fetchColumn ();
	}
	function searchForResultsAndSort($query_term, $sort_col = "rank", $num_returned = 25, $offset = 0) {
		$query_term = SQLite3::escapeString ( $query_term );
		switch ($sort_col) {
			case "rank" :
				$sort_col = "album.rank";
				$sec_sort = "album.title";
				break;
			case "year" :
				$sort_col = "album.year";
				$sec_sort = "album.rank";
				break;
			case "artist" :
				$sort_col = "artist.artist_name";
				$sec_sort = "album.rank";
				break;
		}
		$sql = "SELECT * FROM album, artist WHERE album.artist_id = artist.artist_id
		AND (album.title LIKE '%$query_term%'
		OR artist.artist_name LIKE '%$query_term%'
		OR album.year = '$query_term') ORDER BY $sort_col ASC, $sec_sort ASC
		LIMIT $num_returned OFFSET $offset";
		$result = $this->query ( $sql );
		if ($result === FALSE) {
			// Only doing this for class. Would never do this in real life
			echo $sql;
			echo '<pre class="bg-danger">';
			print_r ( $this->errorInfo () );
			echo '</pre>';
			return array ();
		}
		$albums = array ();
		foreach ( $result as $row ) {
			$albums [] = Album::getAlbumFromRow ( $row );
		}
		return $albums;
	}
	function getAlbumDetails($id) {
		$sql = "SELECT * FROM album, artist 
				WHERE album.artist_id = artist.artist_id AND album.album_id = $id";
		$result = $this->query ( $sql );
		if ($result === FALSE) {
			// Only doing this for class. Would never do this in real life
			echo $sql;
			echo '<pre class="bg-danger">';
			print_r ( $this->errorInfo () );
			echo '</pre>';
			return NULL;
		}
		return Album::getAlbumFromRow ( $result->fetch () );
	}
	function updateArtist($artist) {
		$sql = "UPDATE artist SET artist_name= ? WHERE artist_id = ?";
		$stm = $this->prepare ( $sql );
		return $stm->execute ( array (
				$artist->name,
				$artist->id 
		) );
	}
	function updateAlbum($album) {
		$sql = "UPDATE album SET title = :title, rank = :rank, year = :year,
				artist_id = :artist_id WHERE album_id = :id";
		$stm = $this->prepare ( $sql );
		return $stm->execute ( array (
				":title" => $album->title,
				":rank" => $album->rank,
				":year" => $album->year,
				":artist_id" => $album->artist->id,
				":id" => $album->id 
		) );
	}
	function deleteAlbum($album) {
		// Anyone see this issues with the way I am deleting?
		$sql = "DELETE FROM album WHERE album_id = $album";
		if ($this->exec ( $sql ) === FALSE) {
			echo '<pre class="bg-danger">';
			print_r ( $this->errorInfo () );
			echo '</pre>';
			return FALSE;
		}
		return TRUE;
	}
}